/*
  複数I/Fの同時使用

  ターゲット:DX/AX/RX/EX/MXシリーズ
*/

#include <stdio.h>
#include <conio.h>
#include <process.h>
#include "dxlib.h"
#include "dxmemmap.h"

// マクロ定義
#define COMPORT1    "\\\\.\\COM4"   // ポート番号
#define COMPORT2    "\\\\.\\COM5"
#define BAUDRATE    (57143)         // ボーレート[bps]
#define OFFSETTIME  (20)            // オフセット[ms]
#define TIMEOUT     (50)            // 受信タイムアウト [ms]
#define TARGETID1   (1)             // 対象となるID
#define TARGETID2   (1)             // 対象となるID

#define MAX_POS     (1023)
#define MIN_POS     (0)

BOOL terminate = FALSE;

BOOL DxGetPosition (TDeviceID dvid, UCHAR id, USHORT *pos, TErrorCode *err) {
  return DX_ReadWordData (dvid, id, ADDRESS_PRESENT_POSITION, (USHORT *)pos, err);
}

BOOL DxSetPosition (TDeviceID dvid, UCHAR id, USHORT pos, TErrorCode *err) {
  return DX_WriteWordData (dvid, id, ADDRESS_GOAL_POSITION, (USHORT)pos, err);
}

unsigned __stdcall thread (void *dum) {
  TDeviceID   dev;
  TErrorCode  err;
  int16_t     pos = 511;
  uint16_t    ppos;
  int         f = 1;

  if ((dev = DX_OpenPort (COMPORT2, BAUDRATE))) {
    printf ("Open %s success\n", COMPORT2);
  } else {
    printf ("Open %s error\n", COMPORT2);
  }

  DX_SetTimeOutOffset (dev, OFFSETTIME);

  while (!terminate) {
    if (DX_Active (dev)) {
      pos += f;
      if (pos >= MAX_POS) { pos = MAX_POS; f *= -1; }
      else if (pos <= MIN_POS) { pos = MIN_POS; f *= -1; }

      if (!DxSetPosition (dev, TARGETID2, pos, &err))
        printf ("\nSetPos (%s, %d) error [%04X]", COMPORT2, TARGETID2, err);
      if (DxGetPosition (dev, TARGETID2, &ppos, &err))
        printf ("\nGetPos (%s, %d) = %d", COMPORT2, TARGETID2, ppos);
      else
        printf ("\nGetPos (%s, %d) error [%04X]", COMPORT2, TARGETID2, err);
    }
    Sleep (10);
  }

  if (dev) DX_ClosePort (dev);

  _endthreadex (0);
  return 0;
}

void main (void) {
  TDeviceID   dev;
  TErrorCode  err;
  HANDLE      mythread;

  // COMPORT2の処理を別スレッドで
  if ((mythread = (HANDLE)_beginthreadex (NULL, 0, &thread, 0, 0, NULL)) == INVALID_HANDLE_VALUE) {
    printf ("beginthread error\n");
  }

  // COMPORT1の処理をmainで
  if ((dev = DX_OpenPort (COMPORT1, BAUDRATE))) {
    printf ("Open %s success\n", COMPORT1);
  } else {
    printf ("Open %s error\n", COMPORT1);
  }

  Sleep (1000);

  int16_t   pos = 511;
  uint16_t  ppos;
  int       f = 1;
  while (!kbhit ()) {
    if (DX_Active (dev)) {
      pos += f;
      if (pos >= MAX_POS) { pos = MAX_POS; f *= -1; }
      else if (pos <= MIN_POS) { pos = MIN_POS; f *= -1; }

      if (!DxSetPosition (dev, TARGETID1, pos, &err))
        printf ("\nSetPos (%s, %d) error [%04X]", COMPORT1, TARGETID1, err);
      if (DxGetPosition (dev, TARGETID1, &ppos, &err))
        printf ("\nGetPos (%s, %d) = %d", COMPORT1, TARGETID1, ppos);
      else
        printf ("\nGetPos (%s, %d) error [%04X]", COMPORT1, TARGETID1, err);
    }
    Sleep(10);
  }

  if (dev) DX_ClosePort (dev);

  if (mythread != INVALID_HANDLE_VALUE) {
    terminate = TRUE;
    WaitForSingleObject (mythread, 1000);
  }

  printf ("Fin\n");
}
