/*
  マルチスレッドで1つのターゲットへの非同期アクセス

  ターゲット:DX/AX/RX/EX/MXシリーズ
*/

#include <stdio.h>
#include <process.h>
#include <conio.h>
#include "dxlib.h"
#include "dxmemmap.h"

// マクロ定義
#define COMPORT     "\\\\.\\COM4"   // ポート番号
#define BAUDRATE    (57143)         // ボーレート[bps]
#define OFFSETTIME  (20)            // オフセット[ms]
#define TARGETID    (1)             // 対象となるID

#define MAX_POS     (1023)
#define MIN_POS     (0)

BOOL terminate = FALSE;

// 位置取得
BOOL DxGetPosition (TDeviceID dvid, uint8_t id, uint16_t *pos, TErrorCode *err) {
  return DX_ReadWordData (dvid, id, ADDRESS_PRESENT_POSITION, (USHORT *)pos, err);
}

// 位置指令
BOOL DxSetPosition (TDeviceID dvid, uint8_t id, uint16_t pos, TErrorCode *err) {
  return DX_WriteWordData (dvid, id, ADDRESS_GOAL_POSITION, (USHORT)pos, err);
}

unsigned __stdcall testThread (void *pdev) {
  TDeviceID   dev = (TDeviceID)pdev;
  TErrorCode  err;
  uint16_t    p;

  while (!terminate && DX_Active (dev)) {
    // 現在位置の読み取り
    DxGetPosition (dev, TARGETID, &p, &err) ? printf ("\rGetPos=%4d     ", p) : printf ("\rGetPos error [$%04X]", err);
  }

  _endthreadex (0);
  return 0;
}

void main (void) {
  TDeviceID   dev;
  TErrorCode  err;
  uint16_t   pos;
  HANDLE    mythread;
  int f = 1;

  if ((dev = DX_OpenPort (COMPORT, BAUDRATE))) {
    DX_SetTimeOutOffset (dev, OFFSETTIME);
    printf("Open success\n");

    // 現在の位置を読み出し
    if (DX_ReadWordData (dev, TARGETID, ADDRESS_PRESENT_POSITION, &pos, &err)) {
      // 読み出し専用スレッド作成
      if ((mythread = (HANDLE)_beginthreadex (NULL, 0, &testThread, (void *)dev, 0, NULL)) == INVALID_HANDLE_VALUE) {
        printf ("Beginthread error\n");
      } else {
        // 何かキーを押すとループを抜ける
        while (!kbhit () && DX_Active (dev)) {
          pos += f;
          if (pos >= MAX_POS) { pos = MAX_POS; f *= -1; }
          else if (pos <= MIN_POS) { pos = MIN_POS; f *= -1; }
          if (!DxSetPosition (dev, TARGETID, pos, &err)) printf ("\nSetPos error [$%04X]\n", err);

          Sleep (1);
        }
        // スレッド終了
        terminate = TRUE;
        WaitForSingleObject (mythread, 2000);
        CloseHandle (mythread);
      }
    }

    DX_ClosePort(dev);
  } else {
    printf("Open error\n");
  }
  printf( "\nFin");
  getchar();
}
