/*
  SYNCインストラクションで複数台へ同時に位置指令

  ターゲット:DX/AX/RX/EX/MXシリーズ
*/

#include <stdio.h>
#include <conio.h>
#include "dxlib.h"
#include "dxmemmap.h"

// マクロ定義
#define COMPORT     "\\\\.\\COM4"   // ポート番号
#define BAUDRATE    (57143)         // ボーレート[bps]
#define OFFSETTIME  (20)            // オフセット[ms]
#define TARGETID    (1)             // 対象となるID

#define MAX_POS     (1023)
#define MIN_POS     (0)

void main (void) {
  int f = 1;
  uint16_t    pos = 511;
  TDeviceID   dev;
  TErrorCode  err;

  if ((dev = DX_OpenPort (COMPORT, BAUDRATE))) {
    DX_SetTimeOutOffset (dev, OFFSETTIME);

    // 何かキーを押すとループを抜ける
    while (!kbhit () && DX_Active (dev)) {
      uint8_t param[100];
      param[0] = ADDRESS_GOAL_POSITION; // アイテムのアドレス
      param[1] = 2;                     // アイテムのサイズ(ここではWORDなので2byte)
      for (int i = 0; i < 16; i++) {    // id=0～15を対象とする
        param[2 + i * 3] = i + 1;       // ID
        param[3 + i * 3] = pos & 0xff;  // 値の下位
        param[4 + i * 3] = pos >> 8;    // 値の上位
      }

      if (DX_WriteSyncData (dev, param, 2 + 3 * 16, &err))
        printf("\rSend success (%4d)  ", pos);
      else
        printf("\rSend error   [$%04X] ", err);

      pos += f;
      if (pos >= MAX_POS) { pos = MAX_POS; f *= -1; }
      else if (pos <= MIN_POS) { pos = MIN_POS; f *= -1; }

      // 5ミリ秒待ち
      Sleep (5);
    }

    DX_ClosePort(dev);
  } else {
    printf("Open error\n");
  }
  printf( "\nFin");
}
